@@ -100,7 +100,9 @@ inline bool IsExpireStatCacheTime(const struct timespec& ts, const time_t& expir
 {
     struct timespec nowts;
     SetStatCacheTime(nowts);
-    return ((ts.tv_sec + expire) < nowts.tv_sec);
+    nowts.tv_sec -= expire;
+
+    return (0 < CompareStatCacheTime(nowts, ts));
 }
 
 //
@@ -247,7 +249,7 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
     AutoLock lock(&StatCache::stat_cache_lock);
 
     stat_cache_t::iterator iter = stat_cache.end();
-    if(overcheck && '/' != strpath[strpath.length() - 1]){
+    if(overcheck && '/' != *strpath.rbegin()){
         strpath += "/";
         iter = stat_cache.find(strpath);
     }
@@ -258,7 +260,7 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
 
     if(iter != stat_cache.end() && (*iter).second){
         stat_cache_entry* ent = (*iter).second;
-        if(!IsExpireTime || !IsExpireStatCacheTime(ent->cache_date, ExpireTime)){
+        if(0 < ent->notruncate || !IsExpireTime || !IsExpireStatCacheTime(ent->cache_date, ExpireTime)){
             if(ent->noobjcache){
                 if(!IsCacheNoObject){
                     // need to delete this cache.
@@ -333,7 +335,7 @@ bool StatCache::IsNoObjectCache(const std::string& key, bool overcheck)
     AutoLock lock(&StatCache::stat_cache_lock);
 
     stat_cache_t::iterator iter = stat_cache.end();
-    if(overcheck && '/' != strpath[strpath.length() - 1]){
+    if(overcheck && '/' != *strpath.rbegin()){
         strpath += "/";
         iter     = stat_cache.find(strpath);
     }
@@ -343,7 +345,8 @@ bool StatCache::IsNoObjectCache(const std::string& key, bool overcheck)
     }
 
     if(iter != stat_cache.end() && (*iter).second) {
-        if(!IsExpireTime || !IsExpireStatCacheTime((*iter).second->cache_date, ExpireTime)){
+        stat_cache_entry* ent = (*iter).second;
+        if(0 < ent->notruncate || !IsExpireTime || !IsExpireStatCacheTime((*iter).second->cache_date, ExpireTime)){
             if((*iter).second->noobjcache){
                 // noobjcache = true means no object.
                 SetStatCacheTime((*iter).second->cache_date);
@@ -418,12 +421,11 @@ bool StatCache::AddStat(const std::string& key, headers_t& meta, bool forcedir,
     // add
     AutoLock lock(&StatCache::stat_cache_lock);
 
-    stat_cache_t::iterator iter = stat_cache.find(key);   // recheck for same key exists
-    if(stat_cache.end() != iter){
-        delete iter->second;
-        stat_cache.erase(iter);
+    std::pair<stat_cache_t::iterator, bool> pair = stat_cache.insert(std::make_pair(key, ent));
+    if(!pair.second){
+        delete pair.first->second;
+        pair.first->second = ent;
     }
-    stat_cache[key] = ent;
 
     // check symbolic link cache
     if(!S_ISLNK(ent->stbuf.st_mode)){
@@ -435,6 +437,47 @@ bool StatCache::AddStat(const std::string& key, headers_t& meta, bool forcedir,
     return true;
 }
 
+// [NOTE]
+// Updates only meta data if cached data exists.
+// And when these are updated, it also updates the cache time.
+//
+bool StatCache::UpdateMetaStats(const std::string& key, headers_t& meta)
+{
+    if(CacheSize < 1){
+        return true;
+    }
+    S3FS_PRN_INFO3("update stat cache entry[path=%s]", key.c_str());
+
+    AutoLock lock(&StatCache::stat_cache_lock);
+    stat_cache_t::iterator iter = stat_cache.find(key);
+    if(stat_cache.end() == iter || !(iter->second)){
+        return true;
+    }
+    stat_cache_entry* ent = iter->second;
+
+    // update only meta keys
+    for(headers_t::iterator metaiter = meta.begin(); metaiter != meta.end(); ++metaiter){
+        std::string tag   = lower(metaiter->first);
+        std::string value = metaiter->second;
+        if(tag == "content-type"){
+            ent->meta[metaiter->first] = value;
+        }else if(tag == "content-length"){
+            ent->meta[metaiter->first] = value;
+        }else if(tag == "etag"){
+            ent->meta[metaiter->first] = value;
+        }else if(tag == "last-modified"){
+            ent->meta[metaiter->first] = value;
+        }else if(is_prefix(tag.c_str(), "x-amz")){
+            ent->meta[tag] = value;      // key is lower case for "x-amz"
+        }
+    }
+
+    // Update time.
+    SetStatCacheTime(ent->cache_date);
+
+    return true;
+}
+
 bool StatCache::AddNoObjectCache(const std::string& key)
 {
     if(!IsCacheNoObject){
@@ -476,12 +519,11 @@ bool StatCache::AddNoObjectCache(const std::string& key)
     // add
     AutoLock lock(&StatCache::stat_cache_lock);
 
-    stat_cache_t::iterator iter = stat_cache.find(key);   // recheck for same key exists
-    if(stat_cache.end() != iter){
-        delete iter->second;
-        stat_cache.erase(iter);
+    std::pair<stat_cache_t::iterator, bool> pair = stat_cache.insert(std::make_pair(key, ent));
+    if(!pair.second){
+        delete pair.first->second;
+        pair.first->second = ent;
     }
-    stat_cache[key] = ent;
 
     // check symbolic link cache
     if(symlink_cache.end() != symlink_cache.find(key)){
@@ -586,7 +628,7 @@ bool StatCache::DelStat(const char* key, bool lock_already_held)
     }
     if(0 < strlen(key) && 0 != strcmp(key, "/")){
         std::string strpath = key;
-        if('/' == strpath[strpath.length() - 1]){
+        if('/' == *strpath.rbegin()){
             // If there is "path" cache, delete it.
             strpath.erase(strpath.length() - 1);
         }else{
@@ -671,12 +713,11 @@ bool StatCache::AddSymlink(const std::string& key, const std::string& value)
     // add
     AutoLock lock(&StatCache::stat_cache_lock);
 
-    symlink_cache_t::iterator iter = symlink_cache.find(key);   // recheck for same key exists
-    if(symlink_cache.end() != iter){
-        delete iter->second;
-        symlink_cache.erase(iter);
+    std::pair<symlink_cache_t::iterator, bool> pair = symlink_cache.insert(std::make_pair(key, ent));
+    if(!pair.second){
+        delete pair.first->second;
+        pair.first->second = ent;
     }
-    symlink_cache[key] = ent;
 
     return true;
 }
@@ -770,21 +811,42 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
     pst->st_blksize = 4096;
 
     // mtime
-    pst->st_mtime = get_mtime(meta);
+    struct timespec mtime = get_mtime(meta);
     if(pst->st_mtime < 0){
         pst->st_mtime = 0L;
+    }else{
+#if defined(__APPLE__)
+        pst->st_mtime = mtime.tv_sec;
+#else
+        pst->st_mtim.tv_sec = mtime.tv_sec;
+        pst->st_mtim.tv_nsec = mtime.tv_nsec;
+#endif
     }
 
     // ctime
-    pst->st_ctime = get_ctime(meta);
+    struct timespec ctime = get_ctime(meta);
     if(pst->st_ctime < 0){
         pst->st_ctime = 0L;
+    }else{
+#if defined(__APPLE__)
+        pst->st_ctime = ctime.tv_sec;
+#else
+        pst->st_ctim.tv_sec = ctime.tv_sec;
+        pst->st_ctim.tv_nsec = ctime.tv_nsec;
+#endif
     }
 
     // atime
-    pst->st_atime = get_atime(meta);
+    struct timespec atime = get_atime(meta);
     if(pst->st_atime < 0){
         pst->st_atime = 0L;
+    }else{
+#if defined(__APPLE__)
+        pst->st_atime = atime.tv_sec;
+#else
+        pst->st_atim.tv_sec = atime.tv_sec;
+        pst->st_atim.tv_nsec = atime.tv_nsec;
+#endif
     }
 
     // size
