@@ -158,6 +158,9 @@ class StatCache
         // Add stat cache
         bool AddStat(const std::string& key, headers_t& meta, bool forcedir = false, bool no_truncate = false);
 
+        // Update meta stats
+        bool UpdateMetaStats(const std::string& key, headers_t& meta);
+
         // Change no truncate flag
         void ChangeNoTruncateFlag(const std::string& key, bool no_truncate);
 
