@@ -21,6 +21,8 @@
 #ifndef S3FS_COMMON_H_
 #define S3FS_COMMON_H_
 
+#include <fcntl.h>
+
 #include "../config.h"
 #include "types.h"
 #include "s3fs_logger.h"
@@ -29,8 +31,9 @@
 // Global variables
 //-------------------------------------------------------------------
 // TODO: namespace these
-extern int64_t        FIVE_GB;
-extern off_t          MIN_MULTIPART_SIZE;
+static const int64_t  FIVE_GB            = 5LL * 1024LL * 1024LL * 1024LL;
+static const off_t    MIN_MULTIPART_SIZE = 5 * 1024 * 1024;
+
 extern bool           foreground;
 extern bool           nomultipart;
 extern bool           pathrequeststyle;
