@@ -21,8 +21,14 @@
 #ifndef S3FS_CURL_H_
 #define S3FS_CURL_H_
 
+#include <cassert>
 #include <curl/curl.h>
+#include <list>
+#include <map>
+#include <strings.h>
+#include <vector>
 
+#include "common.h"
 #include "curl_handlerpool.h"
 #include "bodydata.h"
 #include "psemaphore.h"
@@ -127,7 +133,7 @@ class S3fsCurl
         static int              retries;
         static bool             is_public_bucket;
         static acl_t            default_acl;
-        static storage_class_t  storage_class;
+        static std::string      storage_class;
         static sseckeylist_t    sseckeys;
         static std::string      ssekmsid;
         static sse_type_t       ssetype;
@@ -178,18 +184,18 @@ class S3fsCurl
         headers_t            responseHeaders;      // header data by HeaderCallback
         BodyData             bodydata;             // body data by WriteMemoryCallback
         BodyData             headdata;             // header data by WriteMemoryCallback
-        volatile long        LastResponseCode;
+        long                 LastResponseCode;
         const unsigned char* postdata;             // use by post method and read callback function.
-        int                  postdata_remaining;   // use by post method and read callback function.
+        off_t                postdata_remaining;   // use by post method and read callback function.
         filepart             partdata;             // use by multipart upload/get object callback
         bool                 is_use_ahbe;          // additional header by extension
         int                  retry_count;          // retry count for multipart
         FILE*                b_infile;             // backup for retrying
         const unsigned char* b_postdata;           // backup for retrying
-        int                  b_postdata_remaining; // backup for retrying
+        off_t                b_postdata_remaining; // backup for retrying
         off_t                b_partdata_startpos;  // backup for retrying
         off_t                b_partdata_size;      // backup for retrying
-        int                  b_ssekey_pos;         // backup for retrying
+        size_t               b_ssekey_pos;         // backup for retrying
         std::string          b_ssevalue;           // backup for retrying
         sse_type_t           b_ssetype;            // backup for retrying
         std::string          b_from;               // backup for retrying(for copy request)
@@ -278,6 +284,7 @@ class S3fsCurl
         bool UploadMultipartPostComplete();
         bool CopyMultipartPostComplete();
         bool MixMultipartPostComplete();
+        int MapPutErrorResponse(int result);
 
     public:
         // class methods
@@ -302,8 +309,8 @@ class S3fsCurl
         static bool IsPublicBucket() { return S3fsCurl::is_public_bucket; }
         static acl_t SetDefaultAcl(acl_t acl);
         static acl_t GetDefaultAcl();
-        static storage_class_t SetStorageClass(storage_class_t storage_class);
-        static storage_class_t GetStorageClass() { return S3fsCurl::storage_class; }
+        static std::string SetStorageClass(const std::string& storage_class);
+        static std::string GetStorageClass() { return S3fsCurl::storage_class; }
         static bool LoadEnvSse() { return (S3fsCurl::LoadEnvSseCKeys() && S3fsCurl::LoadEnvSseKmsid()); }
         static sse_type_t SetSseType(sse_type_t type);
         static sse_type_t GetSseType() { return S3fsCurl::ssetype; }
@@ -317,8 +324,8 @@ class S3fsCurl
         static bool IsSetSseKmsId() { return !S3fsCurl::ssekmsid.empty(); }
         static const char* GetSseKmsId() { return S3fsCurl::ssekmsid.c_str(); }
         static bool GetSseKey(std::string& md5, std::string& ssekey);
-        static bool GetSseKeyMd5(int pos, std::string& md5);
-        static int GetSseKeyCount();
+        static bool GetSseKeyMd5(size_t pos, std::string& md5);
+        static size_t GetSseKeyCount();
         static bool SetContentMd5(bool flag);
         static bool SetVerbose(bool flag);
         static bool GetVerbose() { return S3fsCurl::is_verbose; }
@@ -328,11 +335,11 @@ class S3fsCurl
         static bool SetAccessKeyWithSessionToken(const char* AccessKeyId, const char* SecretAccessKey, const char * SessionToken);
         static bool IsSetAccessKeyID()
         {
-            return (0 < S3fsCurl::AWSAccessKeyId.size());
+            return !S3fsCurl::AWSAccessKeyId.empty();
         }
         static bool IsSetAccessKeys()
         {
-            return (0 < S3fsCurl::IAM_role.size() || ((0 < S3fsCurl::AWSAccessKeyId.size() || S3fsCurl::is_ibm_iam_auth) && 0 < S3fsCurl::AWSSecretAccessKey.size()));
+            return !S3fsCurl::IAM_role.empty() || ((!S3fsCurl::AWSAccessKeyId.empty() || S3fsCurl::is_ibm_iam_auth) && !S3fsCurl::AWSSecretAccessKey.empty());
         }
         static long SetSslVerifyHostname(long value);
         static long GetSslVerifyHostname() { return S3fsCurl::ssl_verify_hostname; }
@@ -375,8 +382,8 @@ class S3fsCurl
         bool GetResponseCode(long& responseCode, bool from_curl_handle = true);
         int RequestPerform(bool dontAddAuthHeaders=false);
         int DeleteRequest(const char* tpath);
-        bool PreHeadRequest(const char* tpath, const char* bpath = NULL, const char* savedpath = NULL, int ssekey_pos = -1);
-        bool PreHeadRequest(const std::string& tpath, const std::string& bpath, const std::string& savedpath, int ssekey_pos = -1) {
+        bool PreHeadRequest(const char* tpath, const char* bpath = NULL, const char* savedpath = NULL, size_t ssekey_pos = -1);
+        bool PreHeadRequest(const std::string& tpath, const std::string& bpath, const std::string& savedpath, size_t ssekey_pos = -1) {
           return PreHeadRequest(tpath.c_str(), bpath.c_str(), savedpath.c_str(), ssekey_pos);
         }
         int HeadRequest(const char* tpath, headers_t& meta);
@@ -392,15 +399,14 @@ class S3fsCurl
         int MultipartListRequest(std::string& body);
         int AbortMultipartUpload(const char* tpath, const std::string& upload_id);
         int MultipartHeadRequest(const char* tpath, off_t size, headers_t& meta, bool is_copy);
-        int MultipartUploadRequest(const char* tpath, headers_t& meta, int fd, bool is_copy);
-        int MultipartUploadRequest(const std::string& upload_id, const char* tpath, int fd, off_t offset, off_t size, etaglist_t& list);
+        int MultipartUploadRequest(const std::string& upload_id, const char* tpath, int fd, off_t offset, off_t size, int part_num, std::string* petag);
         int MultipartRenameRequest(const char* from, const char* to, headers_t& meta, off_t size);
 
         // methods(variables)
         CURL* GetCurlHandle() const { return hCurl; }
         std::string GetPath() const { return path; }
         std::string GetBasePath() const { return base_path; }
-        std::string GetSpacialSavedPath() const { return saved_path; }
+        std::string GetSpecialSavedPath() const { return saved_path; }
         std::string GetUrl() const { return url; }
         std::string GetOp() const { return op; }
         headers_t* GetResponseHeaders() { return &responseHeaders; }
@@ -415,7 +421,7 @@ class S3fsCurl
         int GetMultipartRetryCount() const { return retry_count; }
         void SetMultipartRetryCount(int retrycnt) { retry_count = retrycnt; }
         bool IsOverMultipartRetryCount() const { return (retry_count >= S3fsCurl::retries); }
-        int GetLastPreHeadSeecKeyPos() const { return b_ssekey_pos; }
+        size_t GetLastPreHeadSeecKeyPos() const { return b_ssekey_pos; }
 };
 
 #endif // S3FS_CURL_H_
