@@ -41,6 +41,7 @@ S3fsMultiCurl::S3fsMultiCurl(int maxParallelism) : maxParallelism(maxParallelism
 #endif
     if (0 != (result = pthread_mutex_init(&completed_tids_lock, &attr))) {
         S3FS_PRN_ERR("could not initialize completed_tids_lock: %i", result);
+        abort();
     }
 }
 
@@ -132,7 +133,7 @@ int S3fsMultiCurl::MultiPerform()
                 } else {
                     int int_retval = (int)(intptr_t)(retval);
                     if (int_retval && !(int_retval == -ENOENT && isMultiHead)) {
-                        S3FS_PRN_WARN("thread failed - rc(%d)", int_retval);
+                        S3FS_PRN_WARN("thread terminated with non-zero return code: %d", int_retval);
                     }
                 }
             }
@@ -168,7 +169,7 @@ int S3fsMultiCurl::MultiPerform()
         } else {
             int int_retval = (int)(intptr_t)(retval);
             if (int_retval && !(int_retval == -ENOENT && isMultiHead)) {
-                S3FS_PRN_WARN("thread failed - rc(%d)", int_retval);
+                S3FS_PRN_WARN("thread terminated with non-zero return code: %d", int_retval);
             }
         }
     }
