@@ -145,7 +145,7 @@ std::string get_sorted_header_keys(const struct curl_slist* list)
             }
             strkey.erase(pos);
         }
-        if(0 < sorted_headers.length()){
+        if(!sorted_headers.empty()){
             sorted_headers += ";";
         }
         sorted_headers += lower(strkey);
@@ -192,9 +192,11 @@ std::string get_canonical_headers(const struct curl_slist* list)
                 // skip empty-value headers (as they are discarded by libcurl)
                 continue;
             }
-            strhead       = strkey + ":" + strval;
+            strhead = strkey;
+            strhead += ":";
+            strhead += strval;
         }else{
-            strhead       = trim(lower(strhead));
+            strhead = trim(lower(strhead));
         }
         canonical_headers += strhead;
         canonical_headers += "\n";
@@ -221,9 +223,11 @@ std::string get_canonical_headers(const struct curl_slist* list, bool only_amz)
                 // skip empty-value headers (as they are discarded by libcurl)
                 continue;
             }
-            strhead       = strkey + ":" + strval;
+            strhead = strkey;
+            strhead += ":";
+            strhead += strval;
         }else{
-            strhead       = trim(lower(strhead));
+            strhead = trim(lower(strhead));
         }
         if(only_amz && strhead.substr(0, 5) != "x-amz"){
             continue;
@@ -254,9 +258,9 @@ std::string prepare_url(const char* url)
     std::string path;
     std::string url_str = std::string(url);
     std::string token = std::string("/") + bucket;
-    int bucket_pos;
-    int bucket_length = token.size();
-    int uri_length = 0;
+    size_t bucket_pos;
+    size_t bucket_length = token.size();
+    size_t uri_length = 0;
 
     if(!strncasecmp(url_str.c_str(), "https://", 8)){
         uri_length = 8;
@@ -313,7 +317,7 @@ bool make_md5_from_binary(const char* pstr, size_t length, std::string& md5)
     }
     // base64 md5
     md5 = s3fs_get_content_md5(fd);
-    if(0 == md5.length()){
+    if(md5.empty()){
         S3FS_PRN_ERR("Failed to make MD5.");
         fclose(fp);
         return false;
@@ -326,14 +330,14 @@ std::string url_to_host(const std::string &url)
 {
     S3FS_PRN_INFO3("url is %s", url.c_str());
 
-    static const std::string http = "http://";
-    static const std::string https = "https://";
+    static const char HTTP[] = "http://";
+    static const char HTTPS[] = "https://";
     std::string hostname;
 
-    if (is_prefix(url.c_str(), http.c_str())) {
-        hostname = url.substr(http.size());
-    } else if (is_prefix(url.c_str(), https.c_str())) {
-        hostname = url.substr(https.size());
+    if (is_prefix(url.c_str(), HTTP)) {
+        hostname = url.substr(sizeof(HTTP) - 1);
+    } else if (is_prefix(url.c_str(), HTTPS)) {
+        hostname = url.substr(sizeof(HTTPS) - 1);
     } else {
         S3FS_PRN_EXIT("url does not begin with http:// or https://");
         abort();
@@ -383,11 +387,11 @@ const char* getCurlDebugHead(curl_infotype type)
 //
 bool etag_equals(std::string s1, std::string s2)
 {
-    if(s1.length() > 1 && s1[0] == '\"' && s1[s1.length() - 1] == '\"'){
+    if(s1.length() > 1 && s1[0] == '\"' && *s1.rbegin() == '\"'){
         s1.erase(s1.size() - 1);
         s1.erase(0, 1);
     }
-    if(s2.length() > 1 && s2[0] == '\"' && s2[s2.length() - 1] == '\"'){
+    if(s2.length() > 1 && s2[0] == '\"' && *s2.rbegin() == '\"'){
         s2.erase(s2.size() - 1);
         s2.erase(0, 1);
     }
