@@ -28,25 +28,12 @@
 #include "common.h"
 #include "s3fs.h"
 #include "fdcache.h"
+#include "fdcache_pseudofd.h"
 #include "s3fs_util.h"
 #include "s3fs_logger.h"
 #include "string_util.h"
 #include "autolock.h"
 
-//------------------------------------------------
-// Symbols
-//------------------------------------------------
-#define TMPFILE_FOR_CHECK_HOLE  "/tmp/.s3fs_hole_check.tmp"
-
-//
-// For cache directory top path
-//
-#if defined(P_tmpdir)
-#define TMPFILE_DIR_0PATH   P_tmpdir
-#else
-#define TMPFILE_DIR_0PATH   "/tmp"
-#endif
-
 //
 // The following symbols are used by FdManager::RawCheckAllCache().
 //
@@ -95,9 +82,11 @@ bool            FdManager::is_lock_init(false);
 std::string     FdManager::cache_dir;
 bool            FdManager::check_cache_dir_exist(false);
 off_t           FdManager::free_disk_space = 0;
+off_t           FdManager::fake_used_disk_space = 0;
 std::string     FdManager::check_cache_output;
 bool            FdManager::checked_lseek(false);
 bool            FdManager::have_lseek_hole(false);
+std::string     FdManager::tmp_dir = "/tmp";
 
 //------------------------------------------------
 // FdManager class methods
@@ -249,17 +238,7 @@ bool FdManager::CheckCacheDirExist()
     if(FdManager::cache_dir.empty()){
         return true;
     }
-    // check the directory
-    struct stat st;
-    if(0 != stat(cache_dir.c_str(), &st)){
-        S3FS_PRN_ERR("could not access to cache directory(%s) by errno(%d).", cache_dir.c_str(), errno);
-        return false;
-    }
-    if(!S_ISDIR(st.st_mode)){
-        S3FS_PRN_ERR("the cache directory(%s) is not directory.", cache_dir.c_str());
-        return false;
-    }
-    return true;
+    return IsDir(&cache_dir);
 }
 
 off_t FdManager::GetEnsureFreeDiskSpace()
@@ -276,6 +255,20 @@ off_t FdManager::SetEnsureFreeDiskSpace(off_t size)
     return old;
 }
 
+bool FdManager::InitFakeUsedDiskSize(off_t fake_freesize)
+{
+    FdManager::fake_used_disk_space = 0;    // At first, clear this value because this value is used in GetFreeDiskSpace.
+
+    off_t actual_freesize = FdManager::GetFreeDiskSpace(NULL);
+
+    if(fake_freesize < actual_freesize){
+        FdManager::fake_used_disk_space = actual_freesize - fake_freesize;
+    }else{
+        FdManager::fake_used_disk_space = 0;
+    }
+    return true;
+}
+
 off_t FdManager::GetFreeDiskSpace(const char* path)
 {
     struct statvfs vfsbuf;
@@ -287,7 +280,7 @@ off_t FdManager::GetFreeDiskSpace(const char* path)
             ctoppath += "/";
         }
     }else{
-        ctoppath = TMPFILE_DIR_0PATH "/";
+        ctoppath = tmp_dir + "/";
     }
     if(path && '\0' != *path){
         ctoppath += path;
@@ -298,7 +291,10 @@ off_t FdManager::GetFreeDiskSpace(const char* path)
         S3FS_PRN_ERR("could not get vfs stat by errno(%d)", errno);
         return 0;
     }
-    return (vfsbuf.f_bavail * vfsbuf.f_frsize);
+
+    off_t actual_freesize = vfsbuf.f_bavail * vfsbuf.f_frsize;
+
+    return (FdManager::fake_used_disk_space < actual_freesize ? (actual_freesize - FdManager::fake_used_disk_space) : 0);
 }
 
 bool FdManager::IsSafeDiskSpace(const char* path, off_t size)
@@ -313,10 +309,14 @@ bool FdManager::HaveLseekHole()
         return FdManager::have_lseek_hole;
     }
 
-    // create tempolary file
-    int fd;
-    if(-1 == (fd = open(TMPFILE_FOR_CHECK_HOLE, O_CREAT|O_RDWR, 0600))){
-        S3FS_PRN_ERR("failed to open tempolary file(%s) - errno(%d)", TMPFILE_FOR_CHECK_HOLE, errno);
+    // create temporary file
+    FILE* ptmpfp;
+    int   fd;
+    if(NULL == (ptmpfp = MakeTempFile()) || -1 == (fd = fileno(ptmpfp))){
+        S3FS_PRN_ERR("failed to open temporary file by errno(%d)", errno);
+        if(ptmpfp){
+            fclose(ptmpfp);
+        }
         FdManager::checked_lseek   = true;
         FdManager::have_lseek_hole = false;
         return FdManager::have_lseek_hole;
@@ -337,13 +337,76 @@ bool FdManager::HaveLseekHole()
         }
     }
     close(fd);
-    unlink(TMPFILE_FOR_CHECK_HOLE);
 
     FdManager::checked_lseek   = true;
     FdManager::have_lseek_hole = result;
     return FdManager::have_lseek_hole;
 }
 
+bool FdManager::SetTmpDir(const char *dir)
+{
+    if(!dir || '\0' == dir[0]){
+        tmp_dir = "/tmp";
+    }else{
+        tmp_dir = dir;
+    }
+    return true;
+}
+
+bool FdManager::IsDir(const std::string* dir)
+{
+    // check the directory
+    struct stat st;
+    if(0 != stat(dir->c_str(), &st)){
+        S3FS_PRN_ERR("could not stat() directory %s by errno(%d).", dir->c_str(), errno);
+        return false;
+    }
+    if(!S_ISDIR(st.st_mode)){
+        S3FS_PRN_ERR("the directory %s is not a directory.", dir->c_str());
+        return false;
+    }
+    return true;
+}
+
+bool FdManager::CheckTmpDirExist()
+{
+    if(FdManager::tmp_dir.empty()){
+        return true;
+    }
+    return IsDir(&tmp_dir);
+}
+
+FILE* FdManager::MakeTempFile() {
+    int fd;
+    char cfn[PATH_MAX];
+    std::string fn = tmp_dir + "/s3fstmp.XXXXXX";
+    strncpy(cfn, fn.c_str(), sizeof(cfn) - 1);
+    cfn[sizeof(cfn) - 1] = '\0';
+
+    fd = mkstemp(cfn);
+    if (-1 == fd) {
+        S3FS_PRN_ERR("failed to create tmp file. errno(%d)", errno);
+        return NULL;
+    }
+    if (-1 == unlink(cfn)) {
+        S3FS_PRN_ERR("failed to delete tmp file. errno(%d)", errno);
+        return NULL;
+    }
+    return fdopen(fd, "rb+");
+}
+
+bool FdManager::HasOpenEntityFd(const char* path)
+{
+    AutoLock auto_lock(&FdManager::fd_manager_lock);
+
+    FdEntity*   ent;
+    int         fd = -1;
+    if(NULL == (ent = FdManager::singleton.GetFdEntity(path, fd, false, true))){
+        return false;
+    }
+    return (0 < ent->GetOpenCount());
+}
+
 //------------------------------------------------
 // FdManager methods
 //------------------------------------------------
@@ -379,7 +442,7 @@ FdManager::~FdManager()
     if(this == FdManager::get()){
         for(fdent_map_t::iterator iter = fent.begin(); fent.end() != iter; ++iter){
             FdEntity* ent = (*iter).second;
-            S3FS_PRN_WARN("To exit with the cache file opened: path=%s, refcnt=%d", ent->GetPath(), ent->GetRefCnt());
+            S3FS_PRN_WARN("To exit with the cache file opened: path=%s, refcnt=%d", ent->GetPath(), ent->GetOpenCount());
             delete ent;
         }
         fent.clear();
@@ -405,32 +468,39 @@ FdManager::~FdManager()
     }
 }
 
-FdEntity* FdManager::GetFdEntity(const char* path, int existfd, bool increase_ref)
+FdEntity* FdManager::GetFdEntity(const char* path, int& existfd, bool newfd, bool lock_already_held)
 {
-    S3FS_PRN_INFO3("[path=%s][fd=%d]", SAFESTRPTR(path), existfd);
+    S3FS_PRN_INFO3("[path=%s][pseudo_fd=%d]", SAFESTRPTR(path), existfd);
 
     if(!path || '\0' == path[0]){
         return NULL;
     }
-    AutoLock auto_lock(&FdManager::fd_manager_lock);
+    AutoLock auto_lock(&FdManager::fd_manager_lock, lock_already_held ? AutoLock::ALREADY_LOCKED : AutoLock::NONE);
 
     fdent_map_t::iterator iter = fent.find(std::string(path));
-    if(fent.end() != iter && (-1 == existfd || (*iter).second->GetFd() == existfd)){
-        if(increase_ref){
-            iter->second->Dup();
+    if(fent.end() != iter && iter->second){
+        if(-1 == existfd){
+            if(newfd){
+                existfd = iter->second->OpenPseudoFd(O_RDWR);    // [NOTE] O_RDWR flags
+            }
+            return iter->second;
+        }else if(iter->second->FindPseudoFd(existfd)){
+            if(newfd){
+                existfd = iter->second->Dup(existfd);
+            }
+            return iter->second;
         }
-        return (*iter).second;
     }
 
     if(-1 != existfd){
         for(iter = fent.begin(); iter != fent.end(); ++iter){
-            if((*iter).second && (*iter).second->GetFd() == existfd){
+            if(iter->second && iter->second->FindPseudoFd(existfd)){
                 // found opened fd in map
-                if(0 == strcmp((*iter).second->GetPath(), path)){
-                    if(increase_ref){
-                        iter->second->Dup();
+                if(0 == strcmp(iter->second->GetPath(), path)){
+                    if(newfd){
+                        existfd = iter->second->Dup(existfd);
                     }
-                    return (*iter).second;
+                    return iter->second;
                 }
                 // found fd, but it is used another file(file descriptor is recycled)
                 // so returns NULL.
@@ -438,24 +508,31 @@ FdEntity* FdManager::GetFdEntity(const char* path, int existfd, bool increase_re
             }
         }
     }
+
+    // If the cache directory is not specified, s3fs opens a temporary file
+    // when the file is opened.
+    if(!FdManager::IsCacheDir()){
+        for(iter = fent.begin(); iter != fent.end(); ++iter){
+            if(iter->second && iter->second->IsOpen() && 0 == strcmp(iter->second->GetPath(), path)){
+                return iter->second;
+            }
+        }
+    }
     return NULL;
 }
 
-FdEntity* FdManager::Open(const char* path, headers_t* pmeta, off_t size, time_t time, bool force_tmpfile, bool is_create, bool no_fd_lock_wait)
+FdEntity* FdManager::Open(int& fd, const char* path, headers_t* pmeta, off_t size, time_t time, int flags, bool force_tmpfile, bool is_create, AutoLock::Type type)
 {
-    S3FS_PRN_DBG("[path=%s][size=%lld][time=%lld]", SAFESTRPTR(path), static_cast<long long>(size), static_cast<long long>(time));
+    S3FS_PRN_DBG("[path=%s][size=%lld][time=%lld][flags=0x%x]", SAFESTRPTR(path), static_cast<long long>(size), static_cast<long long>(time), flags);
 
     if(!path || '\0' == path[0]){
         return NULL;
     }
-    bool close = false;
-    FdEntity* ent;
 
     AutoLock auto_lock(&FdManager::fd_manager_lock);
 
     // search in mapping by key(path)
     fdent_map_t::iterator iter = fent.find(std::string(path));
-
     if(fent.end() == iter && !force_tmpfile && !FdManager::IsCacheDir()){
         // If the cache directory is not specified, s3fs opens a temporary file
         // when the file is opened.
@@ -463,21 +540,30 @@ FdEntity* FdManager::Open(const char* path, headers_t* pmeta, off_t size, time_t
         // search a entity in all which opened the temporary file.
         //
         for(iter = fent.begin(); iter != fent.end(); ++iter){
-            if((*iter).second && (*iter).second->IsOpen() && 0 == strcmp((*iter).second->GetPath(), path)){
+            if(iter->second && iter->second->IsOpen() && 0 == strcmp(iter->second->GetPath(), path)){
                 break;      // found opened fd in mapping
             }
         }
     }
 
+    FdEntity* ent;
     if(fent.end() != iter){
         // found
-        ent = (*iter).second;
-        ent->Dup();
+        ent = iter->second;
+
         if(ent->IsModified()){
-            // If the file is being modified, it will not be resized.
-            size = -1;
+            // If the file is being modified and it's size is larger than size parameter, it will not be resized.
+            off_t cur_size = 0;
+            if(ent->GetSize(cur_size) && size <= cur_size){
+                size = -1;
+            }
+        }
+
+        // (re)open
+        if(-1 == (fd = ent->Open(pmeta, size, time, flags, type))){
+            S3FS_PRN_ERR("failed to (re)open and create new pseudo fd for path(%s).", path);
+            return NULL;
         }
-        close = true;
 
     }else if(is_create){
         // not found
@@ -489,6 +575,12 @@ FdEntity* FdManager::Open(const char* path, headers_t* pmeta, off_t size, time_t
         // make new obj
         ent = new FdEntity(path, cache_path.c_str());
 
+        // open
+        if(-1 == (fd = ent->Open(pmeta, size, time, flags, type))){
+            delete ent;
+            return NULL;
+        }
+
         if(!cache_path.empty()){
             // using cache
             fent[std::string(path)] = ent;
@@ -507,45 +599,40 @@ FdEntity* FdManager::Open(const char* path, headers_t* pmeta, off_t size, time_t
     }else{
         return NULL;
     }
-
-    // open
-    if(0 != ent->Open(pmeta, size, time, no_fd_lock_wait)){
-        if(close){
-            ent->Close();
-        }
-        return NULL;
-    }
-    if(close){
-        ent->Close();
-    }
     return ent;
 }
 
-FdEntity* FdManager::ExistOpen(const char* path, int existfd, bool ignore_existfd)
+// [NOTE]
+// This method does not create a new pseudo fd.
+// It just finds existfd and returns the corresponding entity.
+//
+FdEntity* FdManager::GetExistFdEntity(const char* path, int existfd)
 {
-    S3FS_PRN_DBG("[path=%s][fd=%d][ignore_existfd=%s]", SAFESTRPTR(path), existfd, ignore_existfd ? "true" : "false");
+    S3FS_PRN_DBG("[path=%s][pseudo_fd=%d]", SAFESTRPTR(path), existfd);
 
-    // search by real path
-    FdEntity* ent = Open(path, NULL, -1, -1, false, false);
+    AutoLock auto_lock(&FdManager::fd_manager_lock);
 
-    if(!ent && (ignore_existfd || (-1 != existfd))){
-        // search from all fdentity because of not using cache.
-        AutoLock auto_lock(&FdManager::fd_manager_lock);
-
-        for(fdent_map_t::iterator iter = fent.begin(); iter != fent.end(); ++iter){
-            if((*iter).second && (*iter).second->IsOpen() && (ignore_existfd || ((*iter).second->GetFd() == existfd))){
-                // found opened fd in map
-                if(0 == strcmp((*iter).second->GetPath(), path)){
-                    ent = (*iter).second;
-                    ent->Dup();
-                }else{
-                    // found fd, but it is used another file(file descriptor is recycled)
-                    // so returns NULL.
-                }
-                break;
-            }
+    // search from all entity.
+    for(fdent_map_t::iterator iter = fent.begin(); iter != fent.end(); ++iter){
+        if(iter->second && iter->second->FindPseudoFd(existfd)){
+            // found existfd in entity
+            return iter->second;
         }
     }
+    // not found entity
+    return NULL;
+}
+
+FdEntity* FdManager::OpenExistFdEntity(const char* path, int& fd, int flags)
+{
+    S3FS_PRN_DBG("[path=%s][flags=0x%x]", SAFESTRPTR(path), flags);
+
+    // search entity by path, and create pseudo fd
+    FdEntity* ent = Open(fd, path, NULL, -1, -1, flags, false, false, AutoLock::NONE);
+    if(!ent){
+        // Not found entity
+        return NULL;
+    }
     return ent;
 }
 
@@ -561,7 +648,7 @@ void FdManager::Rename(const std::string &from, const std::string &to)
         // search a entity in all which opened the temporary file.
         //
         for(iter = fent.begin(); iter != fent.end(); ++iter){
-            if((*iter).second && (*iter).second->IsOpen() && 0 == strcmp((*iter).second->GetPath(), from.c_str())){
+            if(iter->second && iter->second->IsOpen() && 0 == strcmp(iter->second->GetPath(), from.c_str())){
                 break;              // found opened fd in mapping
             }
         }
@@ -571,7 +658,7 @@ void FdManager::Rename(const std::string &from, const std::string &to)
         // found
         S3FS_PRN_DBG("[from=%s][to=%s]", from.c_str(), to.c_str());
 
-        FdEntity* ent = (*iter).second;
+        FdEntity* ent = iter->second;
 
         // retrieve old fd entity from map
         fent.erase(iter);
@@ -588,34 +675,33 @@ void FdManager::Rename(const std::string &from, const std::string &to)
     }
 }
 
-bool FdManager::Close(FdEntity* ent)
+bool FdManager::Close(FdEntity* ent, int fd)
 {
-    S3FS_PRN_DBG("[ent->file=%s][ent->fd=%d]", ent ? ent->GetPath() : "", ent ? ent->GetFd() : -1);
+    S3FS_PRN_DBG("[ent->file=%s][pseudo_fd=%d]", ent ? ent->GetPath() : "", fd);
 
-    if(!ent){
+    if(!ent || -1 == fd){
         return true;  // returns success
     }
-
     AutoLock auto_lock(&FdManager::fd_manager_lock);
 
     for(fdent_map_t::iterator iter = fent.begin(); iter != fent.end(); ++iter){
-        if((*iter).second == ent){
-            ent->Close();
+        if(iter->second == ent){
+            ent->Close(fd);
             if(!ent->IsOpen()){
                 // remove found entity from map.
                 fent.erase(iter++);
 
                 // check another key name for entity value to be on the safe side
                 for(; iter != fent.end(); ){
-                    if((*iter).second == ent){
+                    if(iter->second == ent){
                         fent.erase(iter++);
                     }else{
                         ++iter;
                     }
                 }
                 delete ent;
-          }
-          return true;
+            }
+            return true;
         }
     }
     return false;
@@ -626,7 +712,7 @@ bool FdManager::ChangeEntityToTempPath(FdEntity* ent, const char* path)
     AutoLock auto_lock(&FdManager::fd_manager_lock);
 
     for(fdent_map_t::iterator iter = fent.begin(); iter != fent.end(); ){
-        if((*iter).second == ent){
+        if(iter->second == ent){
             fent.erase(iter++);
 
             std::string tmppath;
@@ -906,7 +992,7 @@ bool FdManager::CheckAllCache()
     }
 
     // print head message
-    S3FS_PRN_CACHE(fp, CACHEDBG_FMT_HEAD, S3fsLog::GetCurrentTime());
+    S3FS_PRN_CACHE(fp, CACHEDBG_FMT_HEAD, S3fsLog::GetCurrentTime().c_str());
 
     // Loop in directory of cache file's stats
     std::string top_path  = CacheFileStat::GetCacheFileStatTopDir();
