@@ -101,7 +101,7 @@ static fdpage_list_t raw_compress_fdpage_list(const fdpage_list_t& pages, bool i
             tmppage = fdpage(iter->offset, iter->bytes, (ignore_load ? default_load : iter->loaded), (ignore_modify ? default_modify : iter->modified));
         }
     }
-    // add lastest area
+    // add last area
     if(!is_first){
         raw_add_compress_fdpage_list(compressed_pages, tmppage, ignore_load, ignore_modify, default_load, default_modify);
     }
@@ -183,10 +183,10 @@ bool PageList::GetSparseFilePages(int fd, size_t file_size, fdpage_list_t& spars
     }
 
     bool is_hole   = false;
-    int  hole_pos  = lseek(fd, 0, SEEK_HOLE);
-    int  data_pos  = lseek(fd, 0, SEEK_DATA);
+    off_t hole_pos = lseek(fd, 0, SEEK_HOLE);
+    off_t data_pos = lseek(fd, 0, SEEK_DATA);
     if(-1 == hole_pos && -1 == data_pos){
-        S3FS_PRN_ERR("Could not find the first position both HOLE and DATA in the file(fd=%d).", fd);
+        S3FS_PRN_ERR("Could not find the first position both HOLE and DATA in the file(physical_fd=%d).", fd);
         return false;
     }else if(-1 == hole_pos){
         is_hole   = false;
@@ -198,7 +198,7 @@ bool PageList::GetSparseFilePages(int fd, size_t file_size, fdpage_list_t& spars
         is_hole   = false;
     }
 
-    for(int cur_pos = 0, next_pos = 0; 0 <= cur_pos; cur_pos = next_pos, is_hole = !is_hole){
+    for(off_t cur_pos = 0, next_pos = 0; 0 <= cur_pos; cur_pos = next_pos, is_hole = !is_hole){
         fdpage page;
         page.offset   = cur_pos;
         page.loaded   = !is_hole;
@@ -231,7 +231,7 @@ bool PageList::CheckZeroAreaInFile(int fd, off_t start, size_t bytes)
         bool    found_bad_data = false;
         ssize_t read_bytes;
         if(-1 == (read_bytes = pread(fd, readbuff, check_bytes, (start + comp_bytes)))){
-            S3FS_PRN_ERR("Something error is occurred in reading %zu bytes at %lld from file(%d).", check_bytes, static_cast<long long int>(start + comp_bytes), fd);
+            S3FS_PRN_ERR("Something error is occurred in reading %zu bytes at %lld from file(physical_fd=%d).", check_bytes, static_cast<long long int>(start + comp_bytes), fd);
             found_bad_data = true;
         }else{
             check_bytes = static_cast<size_t>(read_bytes);
@@ -509,10 +509,20 @@ bool PageList::FindUnloadedPage(off_t start, off_t& resstart, off_t& ressize) co
     return false;
 }
 
-off_t PageList::GetTotalUnloadedPageSize(off_t start, off_t size) const
+// [NOTE]
+// Accumulates the range of unload that is smaller than the Limit size.
+// If you want to integrate all unload ranges, set the limit size to 0.
+//
+off_t PageList::GetTotalUnloadedPageSize(off_t start, off_t size, off_t limit_size) const
 {
-    off_t restsize = 0;
+    // If size is 0, it means loading to end.
+    if(0 == size){
+        if(start < Size()){
+            size = Size() - start;
+        }
+    }
     off_t next     = start + size;
+    off_t restsize = 0;
     for(fdpage_list_t::const_iterator iter = pages.begin(); iter != pages.end(); ++iter){
         if(iter->next() <= start){
             continue;
@@ -537,12 +547,14 @@ off_t PageList::GetTotalUnloadedPageSize(off_t start, off_t size) const
                 tmpsize = next - iter->offset;
             }
         }
-        restsize += tmpsize;
+        if(0 == limit_size || tmpsize < limit_size){
+            restsize += tmpsize;
+        }
     }
     return restsize;
 }
 
-int PageList::GetUnloadedPages(fdpage_list_t& unloaded_list, off_t start, off_t size) const
+size_t PageList::GetUnloadedPages(fdpage_list_t& unloaded_list, off_t start, off_t size) const
 {
     // If size is 0, it means loading to end.
     if(0 == size){
@@ -673,7 +685,7 @@ bool PageList::GetPageListsForMultipartUpload(fdpage_list_t& dlpages, fdpage_lis
             }
         }
     }
-    // lastest area
+    // last area
     if(0 < prev_page.bytes){
         mixupload_pages.push_back(prev_page);
     }
@@ -801,7 +813,7 @@ bool PageList::Serialize(CacheFileStat& file, bool is_output, ino_t inode)
             return true;
         }
         char* ptmp = new char[st.st_size + 1];
-        int result;
+        ssize_t result;
         // read from file
         if(0 >= (result = pread(file.GetFd(), ptmp, st.st_size, 0))){
             S3FS_PRN_ERR("failed to read stats(%d)", errno);
@@ -941,7 +953,7 @@ bool PageList::CompareSparseFile(int fd, size_t file_size, fdpage_list_t& err_ar
     // are assigned to any holes.
     fdpage_list_t sparse_list;
     if(!PageList::GetSparseFilePages(fd, file_size, sparse_list)){
-        S3FS_PRN_ERR("Something error is occurred in parsing hole/data of the cache file(%d).", fd);
+        S3FS_PRN_ERR("Something error is occurred in parsing hole/data of the cache file(physical_fd=%d).", fd);
 
         fdpage page(0, static_cast<off_t>(file_size), false, false);
         err_area_list.push_back(page);
