@@ -70,6 +70,9 @@ static const char help_string[] =
     "   retries (default=\"5\")\n"
     "      - number of times to retry a failed S3 transaction\n"
     "\n"
+    "   tmpdir (default=\"/tmp\")\n"
+    "      - local folder for temporary files.\n"
+    "\n"
     "   use_cache (default=\"\" which means disabled)\n"
     "      - local folder to use for local file cache\n"
     "\n"
@@ -337,7 +340,7 @@ static const char help_string[] =
     "      In this mode, the AWSAccessKey and AWSSecretKey will be used as\n"
     "      IBM's Service-Instance-ID and APIKey, respectively.\n"
     "\n"
-    "   ibm_iam_endpoint (default is https://iam.bluemix.net)\n"
+    "   ibm_iam_endpoint (default is https://iam.cloud.ibm.com)\n"
     "      - sets the URL to use for IBM IAM authentication.\n"
     "\n"
     "   use_xattr (default is not handling the extended attribute)\n"
@@ -558,6 +561,12 @@ void show_version()
     VERSION, COMMIT_HASH_VAL, s3fs_crypt_lib_name());
 }
 
+const char* short_version()
+{
+    static const char short_ver[] = "s3fs version " VERSION "(" COMMIT_HASH_VAL ")";
+    return short_ver;
+}
+
 /*
 * Local variables:
 * tab-width: 4
