@@ -29,7 +29,7 @@
 //-------------------------------------------------------------------
 // Variables
 //-------------------------------------------------------------------
-static const char* c_strErrorObjectName = "FILE or SUBDIR in DIR";
+static const char c_strErrorObjectName[] = "FILE or SUBDIR in DIR";
 
 //-------------------------------------------------------------------
 // Functions
@@ -109,7 +109,7 @@ static xmlChar* get_prefix(xmlDocPtr doc)
     return get_base_exp(doc, "Prefix");
 }
 
-xmlChar* get_next_contination_token(xmlDocPtr doc)
+xmlChar* get_next_continuation_token(xmlDocPtr doc)
 {
     return get_base_exp(doc, "NextContinuationToken");
 }
@@ -175,7 +175,7 @@ static char* get_object_name(xmlDocPtr doc, xmlNodePtr node, const char* path)
                 if(strlen(dirpath) > strlen(basepath)){
                     withdirname = &dirpath[strlen(basepath)];
                 }
-                if(0 < withdirname.length() && '/' != withdirname[withdirname.length() - 1]){
+                if(!withdirname.empty() && '/' != *withdirname.rbegin()){
                     withdirname += "/";
                 }
                 withdirname += mybname;
@@ -378,7 +378,7 @@ int append_objects_from_xml_ex(const char* path, xmlDocPtr doc, xmlXPathContextP
                     xmlXPathFreeObject(ETag);
                 }
             }
-            if(!head.insert(name, (0 < stretag.length() ? stretag.c_str() : NULL), is_dir)){
+            if(!head.insert(name, (!stretag.empty() ? stretag.c_str() : NULL), is_dir)){
                 S3FS_PRN_ERR("insert_object returns with error.");
                 xmlXPathFreeObject(key);
                 xmlXPathFreeObject(contents_xp);
@@ -458,7 +458,7 @@ bool simple_parse_xml(const char* data, size_t len, const char* key, std::string
     value.clear();
 
     xmlDocPtr doc;
-    if(NULL == (doc = xmlReadMemory(data, len, "", NULL, 0))){
+    if(NULL == (doc = xmlReadMemory(data, static_cast<int>(len), "", NULL, 0))){
         return false;
     }
 
