@@ -36,7 +36,7 @@
 bool is_truncated(xmlDocPtr doc);
 int append_objects_from_xml_ex(const char* path, xmlDocPtr doc, xmlXPathContextPtr ctx, const char* ex_contents, const char* ex_key, const char* ex_etag, int isCPrefix, S3ObjList& head);
 int append_objects_from_xml(const char* path, xmlDocPtr doc, S3ObjList& head);
-xmlChar* get_next_contination_token(xmlDocPtr doc);
+xmlChar* get_next_continuation_token(xmlDocPtr doc);
 xmlChar* get_next_marker(xmlDocPtr doc);
 bool get_incomp_mpu_list(xmlDocPtr doc, incomp_mpu_list_t& list);
 
