@@ -54,11 +54,11 @@ bool S3ObjList::insert(const char* name, const char* etag, bool is_dir)
         newname = orgname;
     }
     if(is_dir){
-        if('/' != newname[newname.length() - 1]){
+        if('/' != *newname.rbegin()){
             newname += "/";
         }
     }else{
-        if('/' == newname[newname.length() - 1]){
+        if('/' == *newname.rbegin()){
             is_dir = true;
         }
     }
@@ -165,7 +165,7 @@ std::string S3ObjList::GetNormalizedName(const char* name) const
     if(NULL == (ps3obj = GetS3Obj(name))){
         return std::string("");
     }
-    if(0 == (ps3obj->normalname).length()){
+    if(ps3obj->normalname.empty()){
         return std::string(name);
     }
     return ps3obj->normalname;
@@ -219,11 +219,11 @@ bool S3ObjList::GetNameList(s3obj_list_t& list, bool OnlyNormalized, bool CutSla
     s3obj_t::const_iterator iter;
 
     for(iter = objects.begin(); objects.end() != iter; ++iter){
-        if(OnlyNormalized && 0 != (*iter).second.normalname.length()){
+        if(OnlyNormalized && !iter->second.normalname.empty()){
             continue;
         }
         std::string name = (*iter).first;
-        if(CutSlash && 1 < name.length() && '/' == name[name.length() - 1]){
+        if(CutSlash && 1 < name.length() && '/' == *name.rbegin()){
             // only "/" std::string is skipped this.
             name.erase(name.length() - 1);
         }
@@ -242,7 +242,7 @@ bool S3ObjList::MakeHierarchizedList(s3obj_list_t& list, bool haveSlash)
 
     for(liter = list.begin(); list.end() != liter; ++liter){
         std::string strtmp = (*liter);
-        if(1 < strtmp.length() && '/' == strtmp[strtmp.length() - 1]){
+        if(1 < strtmp.length() && '/' == *strtmp.rbegin()){
             strtmp.erase(strtmp.length() - 1);
         }
         h_map[strtmp] = true;
@@ -250,7 +250,7 @@ bool S3ObjList::MakeHierarchizedList(s3obj_list_t& list, bool haveSlash)
         // check hierarchized directory
         for(std::string::size_type pos = strtmp.find_last_of('/'); std::string::npos != pos; pos = strtmp.find_last_of('/')){
             strtmp.erase(pos);
-            if(0 == strtmp.length() || "/" == strtmp){
+            if(strtmp.empty() || "/" == strtmp){
                 break;
             }
             if(h_map.end() == h_map.find(strtmp)){
