@@ -23,6 +23,7 @@
 #include <cstring>
 #include <cerrno>
 #include <climits>
+#include <iomanip>
 
 #include <stdexcept>
 #include <sstream>
@@ -32,9 +33,9 @@
 #include "string_util.h"
 
 //-------------------------------------------------------------------
-// Gloval variables
+// Global variables
 //-------------------------------------------------------------------
-const std::string SPACES = " \t\r\n";
+const char SPACES[] = " \t\r\n";
 
 //-------------------------------------------------------------------
 // Templates
@@ -55,6 +56,15 @@ template std::string str(unsigned long value);
 template std::string str(long long value);
 template std::string str(unsigned long long value);
 
+template<> std::string str(struct timespec value) {
+    std::ostringstream s;
+    s << value.tv_sec;
+    if(value.tv_nsec != 0){
+        s << "." << std::setfill('0') << std::setw(9) << value.tv_nsec;
+    }
+    return s.str();
+}
+
 //-------------------------------------------------------------------
 // Functions
 //-------------------------------------------------------------------
@@ -98,13 +108,13 @@ std::string lower(std::string s)
     return s;
 }
 
-std::string trim_left(const std::string &s, const std::string &t /* = SPACES */)
+std::string trim_left(const std::string &s, const char *t /* = SPACES */)
 {
     std::string d(s);
     return d.erase(0, s.find_first_not_of(t));
 }
 
-std::string trim_right(const std::string &s, const std::string &t /* = SPACES */)
+std::string trim_right(const std::string &s, const char *t /* = SPACES */)
 {
     std::string d(s);
     std::string::size_type i(d.find_last_not_of(t));
@@ -115,7 +125,7 @@ std::string trim_right(const std::string &s, const std::string &t /* = SPACES */
     }
 }
 
-std::string trim(const std::string &s, const std::string &t /* = SPACES */)
+std::string trim(const std::string &s, const char *t /* = SPACES */)
 {
     return trim_left(trim_right(s, t), t);
 }
@@ -142,7 +152,7 @@ std::string urlEncode(const std::string &s)
             result += c;
         }else{
             result += "%";
-            result += s3fs_hex(&c, 1, false);
+            result += s3fs_hex_upper(&c, 1);
         }
     }
     return result;
@@ -172,7 +182,7 @@ std::string urlEncode2(const std::string &s)
             result += c;
         }else{
             result += "%";
-            result += s3fs_hex(&c, 1, false);
+            result += s3fs_hex_upper(&c, 1);
         }
     }
     return result;
@@ -352,12 +362,8 @@ bool convert_unixtime_from_option_arg(const char* argv, time_t& unixtime)
     return true;
 }
 
-std::string s3fs_hex(const unsigned char* input, size_t length, bool lower)
+static std::string s3fs_hex(const unsigned char* input, size_t length, const char *hexAlphabet)
 {
-    static const char hexLower[] = "0123456789abcdef";
-    static const char hexUpper[] = "0123456789ABCDEF";
-
-    const char* hexAlphabet = (lower ? hexLower : hexUpper);
     std::string hex;
     for(size_t pos = 0; pos < length; ++pos){
         hex += hexAlphabet[input[pos] / 16];
@@ -366,9 +372,19 @@ std::string s3fs_hex(const unsigned char* input, size_t length, bool lower)
     return hex;
 }
 
+std::string s3fs_hex_lower(const unsigned char* input, size_t length)
+{
+    return s3fs_hex(input, length, "0123456789abcdef");
+}
+
+std::string s3fs_hex_upper(const unsigned char* input, size_t length)
+{
+    return s3fs_hex(input, length, "0123456789ABCDEF");
+}
+
 char* s3fs_base64(const unsigned char* input, size_t length)
 {
-    static const char* base = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
+    static const char base[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
     char* result;
 
     if(!input || 0 == length){
@@ -460,7 +476,7 @@ unsigned char* s3fs_decode64(const char* input, size_t* plength)
 
 // Base location for transform.  The range 0xE000 - 0xF8ff
 // is a private range, se use the start of this range.
-static unsigned int escape_base = 0xe000;
+static const unsigned int escape_base = 0xe000;
 
 // encode bytes into wobbly utf8.  
 // 'result' can be null. returns true if transform was needed.
