@@ -24,6 +24,20 @@ set -o pipefail
 
 source test-utils.sh
 
+function test_create_empty_file {
+    describe "Testing creating an empty file ..."
+
+    OBJECT_NAME="$(basename $PWD)/${TEST_TEXT_FILE}"
+
+    touch ${TEST_TEXT_FILE}
+
+    check_file_size "${TEST_TEXT_FILE}" 0
+
+    aws_cli s3api head-object --bucket "${TEST_BUCKET_1}" --key "${OBJECT_NAME}"
+
+    rm_test_file
+}
+
 function test_append_file {
     describe "Testing append to file ..."
     TEST_INPUT="echo ${TEST_TEXT} to ${TEST_TEXT_FILE}"
@@ -125,6 +139,19 @@ function test_mv_file {
     rm_test_file $ALT_TEST_TEXT_FILE
 }
 
+function test_mv_to_exist_file {
+    describe "Testing mv file to exist file function ..."
+
+    BIG_MV_FILE_BLOCK_SIZE=$((BIG_FILE_BLOCK_SIZE + 1))
+
+    dd if=/dev/urandom of="${BIG_FILE}" bs=${BIG_FILE_BLOCK_SIZE} count=${BIG_FILE_COUNT}
+    dd if=/dev/urandom of="${BIG_FILE}-mv" bs=${BIG_MV_FILE_BLOCK_SIZE} count=${BIG_FILE_COUNT}
+
+    mv ${BIG_FILE} ${BIG_FILE}-mv
+
+    rm_test_file "${BIG_FILE}-mv"
+}
+
 function test_mv_empty_directory {
     describe "Testing mv directory function ..."
     if [ -e $TEST_DIR ]; then
@@ -610,6 +637,20 @@ function test_multipart_mix {
     rm_test_file "${BIG_FILE}-mix"
 }
 
+function test_utimens_during_multipart {
+    describe "Testing utimens calling during multipart copy ..."
+
+    dd if=/dev/urandom of="${TEMP_DIR}/${BIG_FILE}" bs=$BIG_FILE_BLOCK_SIZE count=$BIG_FILE_COUNT
+
+    cp ${TEMP_DIR}/${BIG_FILE} ${BIG_FILE}
+
+    # The second copy of the "-p" option calls utimens during multipart upload.
+    cp -p ${TEMP_DIR}/${BIG_FILE} ${BIG_FILE}
+
+    rm -f "${TEMP_DIR}/${BIG_FILE}"
+    rm_test_file "${BIG_FILE}"
+}
+
 function test_special_characters {
     describe "Testing special characters ..."
 
@@ -968,7 +1009,7 @@ function test_update_directory_time() {
     fi
 
     #
-    # mv -> update ctime, not update atime/mtime for taget directory
+    # mv -> update ctime, not update atime/mtime for target directory
     #       not update any for sub-directory and a file
     #
     TIME_TEST_DIR=timetestdir
@@ -1404,7 +1445,7 @@ function test_ensurespace_move_file() {
         return 1
     fi
     if [ ${MOVED_FILE_LENGTH} -ne ${BIG_FILE_LENGTH} ]; then
-        echo "Failed to move file with file length"
+        echo "Failed to move file with file length: ${MOVED_FILE_LENGTH} ${BIG_FILE_LENGTH}"
         return 1
     fi
 
@@ -1425,11 +1466,13 @@ function add_all_tests {
     if ! ps u $S3FS_PID | grep -q ensure_diskfree && ! uname | grep -q Darwin; then
         add_tests test_clean_up_cache
     fi
+    add_tests test_create_empty_file
     add_tests test_append_file
     add_tests test_truncate_file
     add_tests test_truncate_upload
     add_tests test_truncate_empty_file
     add_tests test_mv_file
+    add_tests test_mv_to_exist_file
     add_tests test_mv_empty_directory
     add_tests test_mv_nonempty_directory
     add_tests test_redirects
@@ -1445,18 +1488,12 @@ function add_all_tests {
     add_tests test_external_modification
     add_tests test_read_external_object
     add_tests test_update_metadata_external_small_object
-    if ! uname | grep -q Darwin; then
-        # [NOTE]
-        # This test is very time consuming on OSX and will not run.
-        # And this test should be no different between OSX and
-        # other OSs. so skip this test on OSX.
-        #
-        add_tests test_update_metadata_external_large_object
-    fi
+    add_tests test_update_metadata_external_large_object
     add_tests test_rename_before_close
     add_tests test_multipart_upload
     add_tests test_multipart_copy
     add_tests test_multipart_mix
+    add_tests test_utimens_during_multipart
     add_tests test_special_characters
     add_tests test_hardlink
     add_tests test_symlink
